@@ -43,9 +43,9 @@
 
 typedef struct {
     cairo_surface_t *dst;
-    cairo_rectangle_int16_t extents;
+    cairo_rectangle_int_t extents;
     cairo_image_surface_t *image;
-    cairo_rectangle_int16_t image_rect;
+    cairo_rectangle_int_t image_rect;
     void *image_extra;
 } fallback_state_t;
 
@@ -55,8 +55,8 @@ typedef struct {
  * Acquire destination image surface needed for an image-based
  * fallback.
  *
- * Return value: CAIRO_INT_STATUS_NOTHING_TO_DO if the extents are not
- * visible, CAIRO_STATUS_SUCCESS if some portion is visible and all
+ * Return value: %CAIRO_INT_STATUS_NOTHING_TO_DO if the extents are not
+ * visible, %CAIRO_STATUS_SUCCESS if some portion is visible and all
  * went well, or some error status otherwise.
  **/
 static cairo_int_status_t
@@ -107,7 +107,7 @@ typedef cairo_status_t (*cairo_draw_func_t) (void                          *clos
 					     cairo_surface_t               *dst,
 					     int                            dst_x,
 					     int                            dst_y,
-					     const cairo_rectangle_int16_t *extents);
+					     const cairo_rectangle_int_t   *extents);
 
 static cairo_status_t
 _create_composite_mask_pattern (cairo_surface_pattern_t       *mask_pattern,
@@ -115,7 +115,7 @@ _create_composite_mask_pattern (cairo_surface_pattern_t       *mask_pattern,
 				cairo_draw_func_t             draw_func,
 				void                          *draw_closure,
 				cairo_surface_t               *dst,
-				const cairo_rectangle_int16_t *extents)
+				const cairo_rectangle_int_t   *extents)
 {
     cairo_surface_t *mask;
     cairo_status_t status;
@@ -125,7 +125,7 @@ _create_composite_mask_pattern (cairo_surface_pattern_t       *mask_pattern,
 					 extents->width,
 					 extents->height);
     if (mask->status)
-	return CAIRO_STATUS_NO_MEMORY;
+	return mask->status;
 
     status = (*draw_func) (draw_closure, CAIRO_OPERATOR_ADD,
 			   NULL, mask,
@@ -160,7 +160,7 @@ _clip_and_composite_with_mask (cairo_clip_t                  *clip,
 			       cairo_draw_func_t              draw_func,
 			       void                          *draw_closure,
 			       cairo_surface_t               *dst,
-			       const cairo_rectangle_int16_t *extents)
+			       const cairo_rectangle_int_t   *extents)
 {
     cairo_surface_pattern_t mask_pattern;
     cairo_status_t status;
@@ -194,7 +194,7 @@ _clip_and_composite_combine (cairo_clip_t                  *clip,
 			     cairo_draw_func_t              draw_func,
 			     void                          *draw_closure,
 			     cairo_surface_t               *dst,
-			     const cairo_rectangle_int16_t *extents)
+			     const cairo_rectangle_int_t   *extents)
 {
     cairo_surface_t *intermediate;
     cairo_surface_pattern_t dst_pattern;
@@ -211,7 +211,7 @@ _clip_and_composite_combine (cairo_clip_t                  *clip,
 						 extents->width,
 						 extents->height);
     if (intermediate->status)
-	return CAIRO_STATUS_NO_MEMORY;
+	return intermediate->status;
 
     /* Initialize the intermediate surface from the destination surface
      */
@@ -273,7 +273,7 @@ _clip_and_composite_combine (cairo_clip_t                  *clip,
     return status;
 }
 
-/* Handles compositing for CAIRO_OPERATOR_SOURCE, which is special; it's
+/* Handles compositing for %CAIRO_OPERATOR_SOURCE, which is special; it's
  * defined as (src IN mask IN clip) ADD (dst OUT (mask IN clip))
  */
 static cairo_status_t
@@ -282,7 +282,7 @@ _clip_and_composite_source (cairo_clip_t                  *clip,
 			    cairo_draw_func_t              draw_func,
 			    void                          *draw_closure,
 			    cairo_surface_t               *dst,
-			    const cairo_rectangle_int16_t *extents)
+			    const cairo_rectangle_int_t   *extents)
 {
     cairo_surface_pattern_t mask_pattern;
     cairo_status_t status;
@@ -323,7 +323,7 @@ _clip_and_composite_source (cairo_clip_t                  *clip,
 }
 
 static int
-_cairo_rectangle_empty (const cairo_rectangle_int16_t *rect)
+_cairo_rectangle_empty (const cairo_rectangle_int_t *rect)
 {
     return rect->width == 0 || rect->height == 0;
 }
@@ -356,9 +356,9 @@ _clip_and_composite (cairo_clip_t                  *clip,
 		     cairo_draw_func_t              draw_func,
 		     void                          *draw_closure,
 		     cairo_surface_t               *dst,
-		     const cairo_rectangle_int16_t *extents)
+		     const cairo_rectangle_int_t   *extents)
 {
-    cairo_pattern_union_t solid_pattern;
+    cairo_solid_pattern_t solid_pattern;
     cairo_status_t status;
 
     if (_cairo_rectangle_empty (extents))
@@ -366,7 +366,7 @@ _clip_and_composite (cairo_clip_t                  *clip,
 	return CAIRO_STATUS_SUCCESS;
 
     if (op == CAIRO_OPERATOR_CLEAR) {
-	_cairo_pattern_init_solid (&solid_pattern.solid, CAIRO_COLOR_WHITE,
+	_cairo_pattern_init_solid (&solid_pattern, CAIRO_COLOR_WHITE,
 				   CAIRO_CONTENT_COLOR);
 	src = &solid_pattern.base;
 	op = CAIRO_OPERATOR_DEST_OUT;
@@ -411,18 +411,18 @@ _composite_trap_region (cairo_clip_t            *clip,
 			cairo_pattern_t         *src,
 			cairo_operator_t         op,
 			cairo_surface_t         *dst,
-			pixman_region16_t       *trap_region,
-			cairo_rectangle_int16_t *extents)
+			cairo_region_t          *trap_region,
+			cairo_rectangle_int_t   *extents)
 {
     cairo_status_t status;
-    cairo_pattern_union_t solid_pattern;
-    cairo_pattern_union_t mask;
-    int num_rects = pixman_region_num_rects (trap_region);
+    cairo_solid_pattern_t solid_pattern;
+    cairo_surface_pattern_t mask;
+    int num_rects = _cairo_region_num_boxes (trap_region);
     unsigned int clip_serial;
     cairo_surface_t *clip_surface = clip ? clip->surface : NULL;
 
     if (clip_surface && op == CAIRO_OPERATOR_CLEAR) {
-	_cairo_pattern_init_solid (&solid_pattern.solid, CAIRO_COLOR_WHITE,
+	_cairo_pattern_init_solid (&solid_pattern, CAIRO_COLOR_WHITE,
 				   CAIRO_CONTENT_COLOR);
 	src = &solid_pattern.base;
 	op = CAIRO_OPERATOR_DEST_OUT;
@@ -444,7 +444,7 @@ _composite_trap_region (cairo_clip_t            *clip,
     }
 
     if (clip_surface)
-	_cairo_pattern_init_for_surface (&mask.surface, clip_surface);
+	_cairo_pattern_init_for_surface (&mask, clip_surface);
 
     status = _cairo_surface_composite (op,
 				       src,
@@ -484,16 +484,16 @@ _composite_traps_draw_func (void                          *closure,
 			    cairo_surface_t               *dst,
 			    int                            dst_x,
 			    int                            dst_y,
-			    const cairo_rectangle_int16_t *extents)
+			    const cairo_rectangle_int_t   *extents)
 {
     cairo_composite_traps_info_t *info = closure;
-    cairo_pattern_union_t pattern;
+    cairo_solid_pattern_t pattern;
     cairo_status_t status;
 
     if (dst_x != 0 || dst_y != 0)
 	_cairo_traps_translate (info->traps, - dst_x, - dst_y);
 
-    _cairo_pattern_init_solid (&pattern.solid, CAIRO_COLOR_WHITE,
+    _cairo_pattern_init_solid (&pattern, CAIRO_COLOR_WHITE,
 			       CAIRO_CONTENT_COLOR);
     if (!src)
 	src = &pattern.base;
@@ -520,23 +520,21 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
 				cairo_antialias_t antialias)
 {
     cairo_status_t status;
-    pixman_region16_t trap_region;
-    pixman_region16_t clear_region;
+    cairo_region_t trap_region;
+    cairo_region_t clear_region;
     cairo_bool_t has_trap_region = FALSE;
     cairo_bool_t has_clear_region = FALSE;
-    cairo_rectangle_int16_t extents;
+    cairo_rectangle_int_t extents;
     cairo_composite_traps_info_t traps_info;
 
-    if (traps->num_traps == 0)
+    if (_cairo_operator_bounded_by_mask (op) && traps->num_traps == 0)
         return CAIRO_STATUS_SUCCESS;
 
     status = _cairo_surface_get_extents (dst, &extents);
-
     if (status)
         return status;
 
     status = _cairo_traps_extract_region (traps, &trap_region);
-
     if (CAIRO_INT_STATUS_UNSUPPORTED == status) {
         has_trap_region = FALSE;
     } else if (status) {
@@ -546,24 +544,26 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
     }
 
     if (_cairo_operator_bounded_by_mask (op)) {
-        cairo_rectangle_int16_t trap_extents;
+        cairo_rectangle_int_t trap_extents;
 
         if (has_trap_region) {
             status = _cairo_clip_intersect_to_region (clip, &trap_region);
-
             if (status)
                 goto out;
 
-            _cairo_region_extents_rectangle (&trap_region, &trap_extents);
+            _cairo_region_get_extents (&trap_region, &trap_extents);
         } else {
             cairo_box_t trap_box;
             _cairo_traps_extents (traps, &trap_box);
             _cairo_box_round_to_rectangle (&trap_box, &trap_extents);
         }
 
-        _cairo_rectangle_intersect (&extents, &trap_extents);
-        status = _cairo_clip_intersect_to_rectangle (clip, &extents);
+        if (! _cairo_rectangle_intersect (&extents, &trap_extents)) {
+	    status = CAIRO_STATUS_SUCCESS;
+	    goto out;
+	}
 
+        status = _cairo_clip_intersect_to_rectangle (clip, &extents);
         if (status)
             goto out;
     } else {
@@ -574,9 +574,7 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
              * _cairo_surface_fill_rectangles() or to drawing with a
              * clip region, then we have an additional region to clear.
              */
-            pixman_region_init_rect (&clear_region,
-                                     extents.x, extents.y,
-                                     extents.width, extents.height);
+            _cairo_region_init_rect (&clear_region, &extents);
 
             has_clear_region = TRUE;
             status = _cairo_clip_intersect_to_region (clip, &clear_region);
@@ -584,16 +582,14 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
             if (status)
                 goto out;
 
-            _cairo_region_extents_rectangle (&clear_region,  &extents);
+            _cairo_region_get_extents (&clear_region, &extents);
 
-            if (PIXMAN_REGION_STATUS_SUCCESS !=
-                pixman_region_subtract (&clear_region, &clear_region, &trap_region)) {
-                status = CAIRO_STATUS_NO_MEMORY;
+            status = _cairo_region_subtract (&clear_region, &clear_region, &trap_region);
+            if (status)
                 goto out;
-            }
 
-            if (!pixman_region_not_empty (&clear_region)) {
-                pixman_region_fini (&clear_region);
+            if (!_cairo_region_not_empty (&clear_region)) {
+                _cairo_region_fini (&clear_region);
                 has_clear_region = FALSE;
             }
         } else {
@@ -667,9 +663,9 @@ _clip_and_composite_trapezoids (cairo_pattern_t *src,
 
 out:
     if (has_trap_region)
-        pixman_region_fini (&trap_region);
+        _cairo_region_fini (&trap_region);
     if (has_clear_region)
-        pixman_region_fini (&clear_region);
+        _cairo_region_fini (&clear_region);
 
     return status;
 }
@@ -680,7 +676,7 @@ _cairo_surface_fallback_paint (cairo_surface_t	*surface,
 			       cairo_pattern_t	*source)
 {
     cairo_status_t status;
-    cairo_rectangle_int16_t extents;
+    cairo_rectangle_int_t extents;
     cairo_box_t box;
     cairo_traps_t traps;
 
@@ -689,26 +685,22 @@ _cairo_surface_fallback_paint (cairo_surface_t	*surface,
 	return status;
 
     if (_cairo_operator_bounded_by_source (op)) {
-	cairo_rectangle_int16_t source_extents;
+	cairo_rectangle_int_t source_extents;
 	status = _cairo_pattern_get_extents (source, &source_extents);
 	if (status)
 	    return status;
 
-	_cairo_rectangle_intersect (&extents, &source_extents);
+	if (! _cairo_rectangle_intersect (&extents, &source_extents))
+	    return CAIRO_STATUS_SUCCESS;
     }
 
     status = _cairo_clip_intersect_to_rectangle (surface->clip, &extents);
     if (status)
 	return status;
 
-    box.p1.x = _cairo_fixed_from_int (extents.x);
-    box.p1.y = _cairo_fixed_from_int (extents.y);
-    box.p2.x = _cairo_fixed_from_int (extents.x + extents.width);
-    box.p2.y = _cairo_fixed_from_int (extents.y + extents.height);
+    _cairo_box_from_rectangle (&box, &extents);
 
-    status = _cairo_traps_init_box (&traps, &box);
-    if (status)
-	return status;
+    _cairo_traps_init_box (&traps, &box);
 
     status = _clip_and_composite_trapezoids (source,
 				             op,
@@ -729,7 +721,7 @@ _cairo_surface_mask_draw_func (void                          *closure,
 			       cairo_surface_t               *dst,
 			       int                            dst_x,
 			       int                            dst_y,
-			       const cairo_rectangle_int16_t *extents)
+			       const cairo_rectangle_int_t *extents)
 {
     cairo_pattern_t *mask = closure;
 
@@ -756,7 +748,7 @@ _cairo_surface_fallback_mask (cairo_surface_t		*surface,
 			      cairo_pattern_t		*mask)
 {
     cairo_status_t status;
-    cairo_rectangle_int16_t extents, source_extents, mask_extents;
+    cairo_rectangle_int_t extents, source_extents, mask_extents;
 
     status = _cairo_surface_get_extents (surface, &extents);
     if (status)
@@ -767,7 +759,8 @@ _cairo_surface_fallback_mask (cairo_surface_t		*surface,
 	if (status)
 	    return status;
 
-	_cairo_rectangle_intersect (&extents, &source_extents);
+	if (! _cairo_rectangle_intersect (&extents, &source_extents))
+	    return CAIRO_STATUS_SUCCESS;
     }
 
     if (_cairo_operator_bounded_by_mask (op)) {
@@ -775,7 +768,8 @@ _cairo_surface_fallback_mask (cairo_surface_t		*surface,
 	if (status)
 	    return status;
 
-	_cairo_rectangle_intersect (&extents, &mask_extents);
+	if (! _cairo_rectangle_intersect (&extents, &mask_extents))
+	    return CAIRO_STATUS_SUCCESS;
     }
 
     status = _cairo_clip_intersect_to_rectangle (surface->clip, &extents);
@@ -806,32 +800,32 @@ _cairo_surface_fallback_stroke (cairo_surface_t		*surface,
     cairo_status_t status;
     cairo_traps_t traps;
     cairo_box_t box;
-    cairo_rectangle_int16_t extents;
+    cairo_rectangle_int_t extents;
 
     status = _cairo_surface_get_extents (surface, &extents);
     if (status)
         return status;
 
     if (_cairo_operator_bounded_by_source (op)) {
-	cairo_rectangle_int16_t source_extents;
+	cairo_rectangle_int_t source_extents;
 	status = _cairo_pattern_get_extents (source, &source_extents);
 	if (status)
 	    return status;
 
-	_cairo_rectangle_intersect (&extents, &source_extents);
+	if (! _cairo_rectangle_intersect (&extents, &source_extents))
+	    return CAIRO_STATUS_SUCCESS;
     }
 
     status = _cairo_clip_intersect_to_rectangle (surface->clip, &extents);
     if (status)
         return status;
 
-    box.p1.x = _cairo_fixed_from_int (extents.x);
-    box.p1.y = _cairo_fixed_from_int (extents.y);
-    box.p2.x = _cairo_fixed_from_int (extents.x + extents.width);
-    box.p2.y = _cairo_fixed_from_int (extents.y + extents.height);
+    if (extents.width == 0 || extents.height == 0)
+	return CAIRO_STATUS_SUCCESS;
+
+    _cairo_box_from_rectangle (&box, &extents);
 
     _cairo_traps_init (&traps);
-
     _cairo_traps_limit (&traps, &box);
 
     status = _cairo_path_fixed_stroke_to_traps (path,
@@ -839,10 +833,8 @@ _cairo_surface_fallback_stroke (cairo_surface_t		*surface,
 						ctm, ctm_inverse,
 						tolerance,
 						&traps);
-    if (status) {
-	_cairo_traps_fini (&traps);
-	return status;
-    }
+    if (status)
+	goto FAIL;
 
     status = _clip_and_composite_trapezoids (source,
 				             op,
@@ -851,6 +843,7 @@ _cairo_surface_fallback_stroke (cairo_surface_t		*surface,
 					     surface->clip,
 					     antialias);
 
+FAIL:
     _cairo_traps_fini (&traps);
 
     return status;
@@ -868,32 +861,33 @@ _cairo_surface_fallback_fill (cairo_surface_t		*surface,
     cairo_status_t status;
     cairo_traps_t traps;
     cairo_box_t box;
-    cairo_rectangle_int16_t extents;
+    cairo_rectangle_int_t extents;
 
     status = _cairo_surface_get_extents (surface, &extents);
     if (status)
         return status;
 
     if (_cairo_operator_bounded_by_source (op)) {
-	cairo_rectangle_int16_t source_extents;
+	cairo_rectangle_int_t source_extents;
+
 	status = _cairo_pattern_get_extents (source, &source_extents);
 	if (status)
 	    return status;
 
-	_cairo_rectangle_intersect (&extents, &source_extents);
+	if (! _cairo_rectangle_intersect (&extents, &source_extents))
+	    return CAIRO_STATUS_SUCCESS;
     }
 
     status = _cairo_clip_intersect_to_rectangle (surface->clip, &extents);
     if (status)
         return status;
 
-    box.p1.x = _cairo_fixed_from_int (extents.x);
-    box.p1.y = _cairo_fixed_from_int (extents.y);
-    box.p2.x = _cairo_fixed_from_int (extents.x + extents.width);
-    box.p2.y = _cairo_fixed_from_int (extents.y + extents.height);
+    if (extents.width == 0 || extents.height == 0)
+	return CAIRO_STATUS_SUCCESS;
+
+    _cairo_box_from_rectangle (&box, &extents);
 
     _cairo_traps_init (&traps);
-
     _cairo_traps_limit (&traps, &box);
 
     status = _cairo_path_fixed_fill_to_traps (path,
@@ -930,10 +924,10 @@ _cairo_surface_old_show_glyphs_draw_func (void                          *closure
 					  cairo_surface_t               *dst,
 					  int                            dst_x,
 					  int                            dst_y,
-					  const cairo_rectangle_int16_t *extents)
+					  const cairo_rectangle_int_t *extents)
 {
     cairo_show_glyphs_info_t *glyph_info = closure;
-    cairo_pattern_union_t pattern;
+    cairo_solid_pattern_t pattern;
     cairo_status_t status;
 
     /* Modifying the glyph array is fine because we know that this function
@@ -950,7 +944,7 @@ _cairo_surface_old_show_glyphs_draw_func (void                          *closure
 	}
     }
 
-    _cairo_pattern_init_solid (&pattern.solid, CAIRO_COLOR_WHITE,
+    _cairo_pattern_init_solid (&pattern, CAIRO_COLOR_WHITE,
 			       CAIRO_CONTENT_COLOR);
     if (!src)
 	src = &pattern.base;
@@ -993,7 +987,7 @@ _cairo_surface_fallback_show_glyphs (cairo_surface_t		*surface,
 				     cairo_scaled_font_t	*scaled_font)
 {
     cairo_status_t status;
-    cairo_rectangle_int16_t extents, glyph_extents;
+    cairo_rectangle_int_t extents;
     cairo_show_glyphs_info_t glyph_info;
 
     status = _cairo_surface_get_extents (surface, &extents);
@@ -1001,6 +995,8 @@ _cairo_surface_fallback_show_glyphs (cairo_surface_t		*surface,
 	return status;
 
     if (_cairo_operator_bounded_by_mask (op)) {
+        cairo_rectangle_int_t glyph_extents;
+
 	status = _cairo_scaled_font_glyph_device_extents (scaled_font,
 							  glyphs,
 							  num_glyphs,
@@ -1008,7 +1004,8 @@ _cairo_surface_fallback_show_glyphs (cairo_surface_t		*surface,
 	if (status)
 	    return status;
 
-	_cairo_rectangle_intersect (&extents, &glyph_extents);
+	if (! _cairo_rectangle_intersect (&extents, &glyph_extents))
+	    return CAIRO_STATUS_SUCCESS;
     }
 
     status = _cairo_clip_intersect_to_rectangle (surface->clip, &extents);
@@ -1035,22 +1032,25 @@ _cairo_surface_fallback_snapshot (cairo_surface_t *surface)
 {
     cairo_surface_t *snapshot;
     cairo_status_t status;
-    cairo_pattern_union_t pattern;
+    cairo_surface_pattern_t pattern;
     cairo_image_surface_t *image;
     void *image_extra;
 
     status = _cairo_surface_acquire_source_image (surface,
 						  &image, &image_extra);
-    if (status != CAIRO_STATUS_SUCCESS)
-	return (cairo_surface_t *) &_cairo_surface_nil;
+    if (status)
+	return _cairo_surface_create_in_error (status);
 
     snapshot = cairo_image_surface_create (image->format,
 					   image->width,
 					   image->height);
-    if (cairo_surface_status (snapshot))
+    if (cairo_surface_status (snapshot)) {
+	_cairo_surface_release_source_image (surface,
+					     image, image_extra);
 	return snapshot;
+    }
 
-    _cairo_pattern_init_for_surface (&pattern.surface, &image->base);
+    _cairo_pattern_init_for_surface (&pattern, &image->base);
 
     status = _cairo_surface_composite (CAIRO_OPERATOR_SOURCE,
 			               &pattern.base,
@@ -1063,14 +1063,14 @@ _cairo_surface_fallback_snapshot (cairo_surface_t *surface)
 				       image->height);
 
     _cairo_pattern_fini (&pattern.base);
+    _cairo_surface_release_source_image (surface,
+					 image, image_extra);
+
     if (status) {
 	cairo_surface_destroy (snapshot);
-	return (cairo_surface_t *) &_cairo_surface_nil;
+	return _cairo_surface_create_in_error (status);
     }
 
-    _cairo_surface_release_source_image (surface,
-					 image, &image_extra);
-
     snapshot->device_transform = surface->device_transform;
     snapshot->device_transform_inverse = surface->device_transform_inverse;
 
@@ -1123,11 +1123,11 @@ cairo_status_t
 _cairo_surface_fallback_fill_rectangles (cairo_surface_t         *surface,
 					 cairo_operator_t	  op,
 					 const cairo_color_t	 *color,
-					 cairo_rectangle_int16_t *rects,
+					 cairo_rectangle_int_t   *rects,
 					 int			  num_rects)
 {
     fallback_state_t state;
-    cairo_rectangle_int16_t *offset_rects = NULL;
+    cairo_rectangle_int_t *offset_rects = NULL;
     cairo_status_t status;
     int x1, y1, x2, y2;
     int i;
@@ -1151,9 +1151,9 @@ _cairo_surface_fallback_fill_rectangles (cairo_surface_t         *surface,
 	if (rects[i].y < y1)
 	    y1 = rects[i].y;
 
-	if (rects[i].x + rects[i].width > x2)
+	if ((int) (rects[i].x + rects[i].width) > x2)
 	    x2 = rects[i].x + rects[i].width;
-	if (rects[i].y + rects[i].height > y2)
+	if ((int) (rects[i].y + rects[i].height) > y2)
 	    y2 = rects[i].y + rects[i].height;
     }
 
@@ -1167,9 +1167,9 @@ _cairo_surface_fallback_fill_rectangles (cairo_surface_t         *surface,
     /* If the fetched image isn't at 0,0, we need to offset the rectangles */
 
     if (state.image_rect.x != 0 || state.image_rect.y != 0) {
-      offset_rects = _cairo_malloc_ab (num_rects, sizeof (cairo_rectangle_int16_t));
+	offset_rects = _cairo_malloc_ab (num_rects, sizeof (cairo_rectangle_int_t));
 	if (offset_rects == NULL) {
-	    status = CAIRO_STATUS_NO_MEMORY;
+	    status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
 	    goto DONE;
 	}
 
@@ -1225,7 +1225,7 @@ _cairo_surface_fallback_composite_trapezoids (cairo_operator_t		op,
     if (state.image_rect.x != 0 || state.image_rect.y != 0) {
 	offset_traps = _cairo_malloc_ab (num_traps, sizeof (cairo_trapezoid_t));
 	if (!offset_traps) {
-	    status = CAIRO_STATUS_NO_MEMORY;
+	    status = _cairo_error (CAIRO_STATUS_NO_MEMORY);
 	    goto DONE;
 	}
 
@@ -1259,6 +1259,8 @@ _cairo_surface_fallback_clone_similar (cairo_surface_t	*surface,
 				       int		 src_y,
 				       int		 width,
 				       int		 height,
+				       int		*clone_offset_x,
+				       int		*clone_offset_y,
 				       cairo_surface_t **clone_out)
 {
     cairo_status_t status;
@@ -1288,9 +1290,11 @@ _cairo_surface_fallback_clone_similar (cairo_surface_t	*surface,
     status = cairo_status (cr);
     cairo_destroy (cr);
 
-    if (status == CAIRO_STATUS_SUCCESS)
+    if (status == CAIRO_STATUS_SUCCESS) {
+	*clone_offset_x = src_x;
+	*clone_offset_y = src_y;
 	*clone_out = new_surface;
-    else
+    } else
 	cairo_surface_destroy (new_surface);
 
     return status;
