@@ -34,10 +34,13 @@
  *	https://bugs.freedesktop.org/show_bug.cgi?id=2729
  */
 
-cairo_test_t test = {
+static cairo_test_draw_function_t draw;
+
+static const cairo_test_t test = {
     "dash-offset-negative",
     "Tests cairo_set_dash with a negative offset",
-    IMAGE_WIDTH, IMAGE_HEIGHT
+    IMAGE_WIDTH, IMAGE_HEIGHT,
+    draw
 };
 
 static cairo_test_status_t
@@ -55,7 +58,7 @@ draw (cairo_t *cr, int width, int height)
 
     /* Basic 1-1 dash pattern */
     cairo_set_dash (cr, dashes, 1, 0.);
-    
+
     cairo_move_to (cr,  1, 2);
     cairo_line_to (cr, 18, 2);
     cairo_stroke (cr);
@@ -69,7 +72,7 @@ draw (cairo_t *cr, int width, int height)
 
     /* Offset dash by 0.5, rather than the path */
     cairo_set_dash (cr, dashes, 1, 0.5);
-    
+
     cairo_move_to (cr,  1, 8);
     cairo_line_to (cr, 18, 8);
     cairo_stroke (cr);
@@ -78,7 +81,7 @@ draw (cairo_t *cr, int width, int height)
 
     /* Basic 1-1 dash pattern dashing */
     cairo_set_dash (cr, dashes, 1, -4);
-    
+
     cairo_move_to (cr,  1, 11);
     cairo_line_to (cr, 18, 11);
     cairo_stroke (cr);
@@ -90,7 +93,7 @@ draw (cairo_t *cr, int width, int height)
 
     /* Offset dash by 0.5 */
     cairo_set_dash (cr, dashes, 1, -3.5);
-    
+
     cairo_move_to (cr,  1, 17);
     cairo_line_to (cr, 18, 17);
     cairo_stroke (cr);
@@ -101,5 +104,5 @@ draw (cairo_t *cr, int width, int height)
 int
 main (void)
 {
-    return cairo_test (&test, draw);
+    return cairo_test (&test);
 }
